@@ -2,6 +2,8 @@ from hashlib import md5
 import io
 import os
 import logging
+
+from .transaction import Transaction
 from .utils import read_block, tokenize, stringify_path
 logger = logging.getLogger('fsspec')
 
@@ -696,9 +698,9 @@ class AbstractFileSystem(up):
 
         Parameters
         ----------
-        path : str
+        path: str
             file location
-        truncate : bool
+        truncate: bool
             If True, always set file size to 0; if False, update timestamp and
             leave file unchanged, if backend allows this
         """
@@ -789,47 +791,6 @@ class AbstractFileSystem(up):
         cls._cache.clear()
 
 
-class Transaction(object):
-    """Filesystem transaction write context
-
-    Gathers files for deferred commit or discard, so that several write
-    operations can be finalized semi-atomically. This works by having this
-    instance as the ``.transaction`` attribute of the given filesystem
-    """
-
-    def __init__(self, fs):
-        """
-        Parameters
-        ----------
-        fs: FileSystem instance
-        """
-        self.fs = fs
-        self.files = []
-
-    def __enter__(self):
-        self.start()
-
-    def __exit__(self, exc_type, exc_val, exc_tb):
-        """End transaction and commit, if exit is not due to exception"""
-        # only commit if there was no exception
-        self.complete(commit=exc_type is None)
-        self.fs._intrans = False
-
-    def start(self):
-        """Start a transaction on this FileSystem"""
-        self.fs._intrans = True
-
-    def complete(self, commit=True):
-        """Finish transaction: commit or discard all deferred files"""
-        for f in self.files:
-            if commit:
-                f.commit()
-            else:
-                f.discard()
-        self.files = []
-        self.fs._intrans = False
-
-
 class AbstractBufferedFile(io.IOBase):
     """Convenient class to derive from to provide buffering
 
@@ -858,7 +819,7 @@ class AbstractBufferedFile(io.IOBase):
         autocommit: bool
             Whether to write to final destination; may only impact what
             happens when file is being closed.
-        cache_type : str
+        cache_type: str
             Caching policy in read mode, one of 'none', 'bytes', 'mmap', see
             the definitions in ``core``.
         kwargs:
@@ -933,9 +894,9 @@ class AbstractBufferedFile(io.IOBase):
 
         Parameters
         ----------
-        loc : int
+        loc: int
             byte location
-        whence : {0, 1, 2}
+        whence: {0, 1, 2}
             from start of file, current location or end of file, resp.
         """
         loc = int(loc)
@@ -964,7 +925,7 @@ class AbstractBufferedFile(io.IOBase):
 
         Parameters
         ----------
-        data : bytes
+        data: bytes
             Set of bytes to be written.
         """
         if self.mode not in {'wb', 'ab'}:
@@ -988,7 +949,7 @@ class AbstractBufferedFile(io.IOBase):
 
         Parameters
         ----------
-        force : bool
+        force: bool
             When closing, write the last block even if it is smaller than
             blocks are allowed to be. Disallows further writing to this file.
         """
@@ -1046,7 +1007,7 @@ class AbstractBufferedFile(io.IOBase):
 
         Parameters
         ----------
-        length : int (-1)
+        length: int (-1)
             Number of bytes to read; if <0, all remaining bytes.
         """
         length = -1 if length is None else int(length)
@@ -1078,9 +1039,9 @@ class AbstractBufferedFile(io.IOBase):
 
         Parameters
         ----------
-        char : bytes
+        char: bytes
             Thing to find
-        blocks : None or int
+        blocks: None or int
             How much to read in each go. Defaults to file blocksize - which may
             mean a new read on every call.
         """
